<?php

/***************************************************************************
 * 
 *  Name: Example Module
 *  URI: http://jcore.net
 *  Description: This is an example module to start your own module with. This module is  UNLICENSED meaning you can do whatever you want with it, please see http://unlicense.org/ or MIT License http://en.wikipedia.org/wiki/MIT_License where unlicense isn't juristicaly recognized.
 *  Author: Istvan Petres
 *  Version: 0.9
 *  Tags: example module, unlicensed
 * 
 ****************************************************************************/

class exampleModule extends modules {
	var $limit = 5;
	
	// Starting from ver 0.2 this has to be defined so we can check 
	// if someone is trying to access this module using a different 
	// path to overcome the path permissions limitation
	//
	// You can define it as an array too if you need more then one
	// path to access this module, for e.g. $adminpath = array('path1', 'path2');
	var $adminPath = 'admin/modules/examplemodule';
	
	function __construct() {
		// Load our own translatin file that will translate this module
		// File name should be examplemodule.mo and it should be in
		// /home/yourclient/public_html/locale/en_US/LC_MESSAGES/
		languages::load('examplemodule');
	}
	
	function __destruct() {
		// Reload the original messages.mo file
		languages::unload('examplemodule');
	}
	
	function installSQL() {
		// Run query to set up our own table, debug is turned on 
		// globally so you don't need to display the query manually
		sql::run(
			"CREATE TABLE IF NOT EXISTS `{examplemoduleitems}` (" .
			" `ID` smallint(5) unsigned NOT NULL auto_increment," .
			" `Title` varchar(255) NOT NULL default ''," .
			" `TimeStamp` timestamp NOT NULL default CURRENT_TIMESTAMP," .
			" `OrderID` mediumint(9) NOT NULL default '0'," .
			" PRIMARY KEY  (`ID`)," .
			" KEY `OrderID` (`OrderID`)" .
			" ) ENGINE=MyISAM;");
		
		if (sql::error())
			return false;
		
		return true;
	}
	
	function installFiles() {
		// It is safe to use http://icons.jcore.net for the icons
		// as this url gets automatically rewriten to your SITE_URL
		// or JCORE_URL when css is loaded
		$css = 
			".example-module {\n" .
			"	margin-bottom: 10px;\n" .
			"}\n" .
			".admin-section-item.as-modules-examplemodule a {\n" .
			"	background-image: url(\"http://icons.jcore.net/48/emblem-cool.png\");\n" .
			"}\n";
		
		// We write our own css file, debug mode is turned on globally
		// so an error is written out if something goes wrong
		return 
			files::save(SITE_PATH.'template/modules/css/examplemodule.css', $css);
	}
	
	function uninstallSQL() {
		// Drop our own table that we created at install
		return
			sql::run(
				" DROP TABLE IF EXISTS `{examplemoduleitems}`;");
	}
	
	function uninstallFiles() {
		// Clean up our own files that have been created by this module
		return 
			files::delete(SITE_PATH.'template/modules/css/examplemodule.css');
	}
	
	// ************************************************   Admin Part
	
	// This function allows you to specify custom quick links that
	// will be only used by this module. Usually the first link
	// is the New Item so you can easily create new items without
	// having to scroll down to the bottom of the page
	function setupAdmin() {
		if ($this->userPermissionType & USER_PERMISSION_TYPE_WRITE)
			favoriteLinks::add(
				_('New Item'), 
				'?path='.admin::path().'#adminform');
		
		favoriteLinks::add(
			__('Pages / Posts'), 
			'?path=admin/content/pages');
	}
	
	// This function is used by the admin to show a little counter 
	// on the module's icon with the number of items in this module
	function countAdminItems() {
		if (!parent::installed($this))
			return 0;
		
		$row = sql::fetch(sql::run(
			" SELECT COUNT(*) AS `Rows`" .
			" FROM `{examplemoduleitems}`" .
			" LIMIT 1"));
		return $row['Rows'];
	}
	
	function verifyAdmin(&$form = null) {
		// Check for outside variables first.
		 
		$reorder = null;
		$orders = null;
		$delete = null;
		$edit = null;
		$id = null;
		
		if (isset($_POST['reordersubmit']))
			$reorder = $_POST['reordersubmit'];
		
		if (isset($_POST['orders']))
			$orders = (array)$_POST['orders'];
		
		if (isset($_GET['delete']))
			$delete = $_GET['delete'];
		
		if (isset($_GET['edit']))
			$edit = $_GET['edit'];
		
		if (isset($_GET['id']))
			$id = (int)$_GET['id'];
		
		// See if one has clicked the Reorder button to reorder the list 
		// and if yes do the reordering. 
		if ($reorder) {
			if (!$orders)
				return false;
			
			foreach($orders as $oid => $ovalue) {
				sql::run("UPDATE `{examplemoduleitems}` " .
					" SET `OrderID` = '".(int)$ovalue."'" .
					" WHERE `ID` = '".(int)$oid."'" .
					($this->userPermissionIDs?
						" AND `ID` IN (".$this->userPermissionIDs.")":
						null));
			}
			
			tooltip::display(
				_("Items have been re-ordered successfully."),
				TOOLTIP_SUCCESS);
			
			return true;
		}
		
		// If the delete icon was clicked on the right we will
		// start deleting the selected item
		if ($delete) {
			if (!$this->delete($id))
				return false;
			
			tooltip::display(
				_("Item has been successfully deleted."),
				TOOLTIP_SUCCESS);
			
			return true;
		}
		
		// This is the form that is used to submit datas.
		// If form->verify() returns false means that something wen't
		// wrong and there are missing fields to be corrected so till
		// the form isn't filled out the right way we don't execute
		// the rest of our code.
		if (!$form->verify())
			return false;
		
		// Form got successfully filled out so we start to look
		// if it is an edit and if yes we update the item.
		if ($edit) {
			if (!$this->edit($id, $form->getPostArray()))
				return false;
				
			tooltip::display(
				_("Item has been successfully updated.")." " .
				"<a href='#adminform'>" .
					__("Edit") .
				"</a>",
				TOOLTIP_SUCCESS);
			
			return true;
		}
		
		// If user has only access to a defined number of ittems we
		// don't allow him to create new items.
		if ($this->userPermissionIDs)
			return false;
		
		// It wasn't an edit request so we add our new item.
		if (!$newid = $this->add($form->getPostArray()))
			return false;
			
		tooltip::display(
			_("Item has been successfully created.")." " .
			"<a href='".url::uri('id, edit, delete') .
				"&amp;id=".$newid."&amp;edit=1#adminform'>" .
				__("Edit") .
			"</a>",
			TOOLTIP_SUCCESS);
		
		$form->reset();
		return true;
	}
	
	function displayAdminItems(&$rows) {
		// Check for outside variables first.
		 
		$id = null;
		
		if (isset($_GET['id']))
			$id = (int)$_GET['id'];
		
		// This is a manually created form just for the ordering, it has
		// nothing to do with the above form. You can drop this if you don't
		// want Ordering feature
		echo
			"<form action='".
				url::uri('id, edit, delete').
				"' method='post'>";
		
		// This is the header of the table where we will list the items
		echo "<table class='list'>" .
				"<thead>" .
				"<tr>" .
					"<th>".__("Order")."</th>" .
					"<th>".__("Title")."</th>";
		
		// Check for permission again as if there is no write access we don't
		// display the edit and delete links		
		if ($this->userPermissionType & USER_PERMISSION_TYPE_WRITE)
			echo
					"<th>".__("Edit")."</th>" .
					"<th>".__("Delete")."</th>";
		
		echo
				"</tr>" .
				"</thead>" .
				"<tbody>";
		
		// Start to dump out the items one by one		
		$i = 0;		
		while($row = sql::fetch($rows)) {
			// Dump out data
			echo 
				"<tr".($i%2?" class='pair'":NULL).">" .
					"<td>" .
						"<input type='text' name='orders[".$row['ID']."]' " .
							"value='".$row['OrderID']."' " .
							"class='order-id-entry' />" .
					"</td>" .
					"<td class='auto-width'>" .
						"<a href='".url::uri('id, edit, delete') .
							"&amp;id=".$row['ID']."' " .
						" class='bold'>" .
						$row['Title'] .
						"</a>" .
					"</td>";
			
			// These are the edit and delete links which are customized from css.
			// If the user doesn't have write access to this module/item we don't display
			// any icons at all.
			if ($this->userPermissionType & USER_PERMISSION_TYPE_WRITE)
				echo
					"<td>" .
						"<a class='admin-link edit' " .
							"title='".__("Edit")."' " .
							"href='".url::uri('id, edit, delete') .
							"&amp;id=".$row['ID']."&amp;edit=1#adminform'>" .
						"</a>" .
					"</td>" .
					"<td>" .
						"<a class='admin-link delete confirm-link' " .
							"title='".__("Delete")."' " .
							"href='".url::uri('id, edit, delete') .
							"&amp;id=".$row['ID']."&amp;delete=1'>" .
						"</a>" .
					"</td>";
			
			echo
				"</tr>";
			
			// When the item is the one selected we show further details with
			// an extra row and dump out each further detail one by one	
			if ($row['ID'] == $id) {
				echo
					"<tr".($i%2?" class='pair'":NULL).">" .
						"<td colspan='10'>" .
							"<div class='admin-content-preview'>";
				
				admin::displayItemData(
					__("Created on"),
					$row['TimeStamp']);
				
				echo
							"</div>" .
						"</td>" .
					"</tr>";
			}
			
			// This is only used to see if a row is pair or not
			$i++;
		}
		
		echo 
				"</tbody>" .
			"</table>" .
			"<br />";
		
		// If no write access to this module by the administrator we don't
		// even enable reordering.
		if ($this->userPermissionType & USER_PERMISSION_TYPE_WRITE)
			echo
				"<input type='submit' name='reordersubmit' value='" .
					__("Reorder")."' class='button' /> " .
				"<input type='reset' name='reset' value='" .
					__("Reset")."' class='button' />" .
				"<div class='clear-both'></div>" .
				"<br />";
		
		// Close the form that is used for reordering items
		echo
			"</form>";
	}
	
	function displayAdmin() {
		// This is how admin title should be displayed.
		// You could use a different one with two titles too like:
		// admin::displayTitle(_('Example'), _('Module'));
		admin::displayTitle(_('Example Module'));
		
		// Check for outside variables first.
		$delete = null;
		$edit = null;
		$id = null;
		
		if (isset($_GET['delete']))
			$delete = $_GET['delete'];
		
		if (isset($_GET['edit']))
			$edit = $_GET['edit'];
		
		if (isset($_GET['id']))
			$id = (int)$_GET['id'];
		
		// Echo out texts if you need like this.
		echo 
			"<p>" .
				_("Text to enter before the admin starts.") .
			"</p>";
		
		// Start admin content
		echo
			"<div class='admin-content'>";
		
		// We create a new form that will be used at the bottom of the 
		// list to submit / edit new items and if $_GET['edit'] is defined
		// we display a different title.
		// We define a separate id for the form (neweditform) because if not
		// and form's title gets translated the form may have a different
		// id and we may not be able to access it from javascript or css
		$form = new form(
				($edit?
					_("Edit Item"):
					_("New Item")),
				'neweditform');
		
		// If we are not in edit mode we will strip out id, delete, limit
		// $_GET arguments as when the form gets submited we don't want to
		// stay at the X page with Y item selected, we want to be redirected
		// to the first page so the new item we just created will show up
		// on the top of the list.
		//
		// If in edit mode we will need to keep the limit, id arguments
		// as that way when the form gets submitted we will still be at the
		// same page and with the same item selected to see the result of our
		// edit
		if (!$edit)
			$form->action = url::uri('id, delete, limit');
		
		// Add a form entry:
		// #1 = Title of the entry
		// #2 = Name of the entry (<input name='xxx'>)
		// #3 = Type of the input (please see form.class.php for the types)
		// #4 = Required (true, false)
		// #5 = Predefined value (<input value='yyy'>)			
		$form->add(
			__('Title'),
			'Title',
			FORM_INPUT_TYPE_TEXT,
			true);
		$form->setStyle('width: 350px;');
		
		// This is a form container that can have other elments and will
		// expand and colapse when clicked on it. If you want to remember
		// if the user has expanded or colapsed the area so when he loads the 
		// form/page next time the container gets automatically expanded / colapsed
		// just define a name for it and that's it (for e.g. replace null with 'ema').
		$form->add(
			__('Additional Options'),
			null,
			FORM_OPEN_FRAME_CONTAINER);
		
		$form->add(
			__('Created on'),
			'TimeStamp',
			FORM_INPUT_TYPE_TIMESTAMP);
		$form->setStyle('width: 170px;');
		$form->setValueType(FORM_VALUE_TYPE_TIMESTAMP);
		
		$form->add(
			__('Order'),
			'OrderID',
			FORM_INPUT_TYPE_TEXT);
		$form->setStyle('width: 50px;');
		$form->setValueType(FORM_VALUE_TYPE_INT);
		
		// We have to close every frame container that we have oppened, 
		// please remember this.
		$form->add(
			null,
			null,
			FORM_CLOSE_FRAME_CONTAINER);
		
		// Add the submit buttons, you can add your owns but this will
		// automatically add Submit and Reset buttons
		$form->addSubmitButtons();
		
		// If in edit mode lets add a Cancel button too.
		if ($edit) {
			$form->add(
				__('Cancel'),
				'cancelbutton',
				 FORM_INPUT_TYPE_BUTTON);
			$form->addAttributes("onclick=\"window.location='".
				str_replace('&amp;', '&', url::uri('id, edit, delete'))."'\"");
		}
		
		// We check if the user has write access to this module cos if not
		// we don't want to run the verify function as he has no rights to 
		// modify anything.
		$selected = null;
		$verifyok = false;
		
		// The $selected here will only be used to check if user has access
		// the the selected item, to prefill the form the $selected will be
		// overwritten below
		if ($id)
			$selected = sql::fetch(sql::run(
				" SELECT `ID` FROM `{examplemoduleitems}`" .
				" WHERE `ID` = '".$id."'" .
				($this->userPermissionIDs?
					" AND `ID` IN (".$this->userPermissionIDs.")":
					null)));
		
		if ($this->userPermissionType & USER_PERMISSION_TYPE_WRITE &&
			((!$edit && !$delete) || $selected))
			$verifyok = $this->verifyAdmin($form);
		
		// Our paging class and we limit the items to 10	
		$paging = new paging(10);
		
		// Get the rows and use the paging's limit as if they are on the
		// second page the paging->limit will be set to reflect that
		$rows = sql::run(
			" SELECT * FROM `{examplemoduleitems}`" .
			" WHERE 1" .
			($this->userPermissionIDs?
				" AND `ID` IN (".$this->userPermissionIDs.")":
				null) .
			" ORDER BY `OrderID`, `TimeStamp` DESC" .
			" LIMIT ".$paging->limit);
	
		// We set the total items for the paging so the paging
		// knows how many pages there are and can show us the next
		// prev pages and page numbers if needed
		$paging->setTotalItems(sql::count());
		
		// Display a simple tooltip if there are no items in our db yet.		
		if ($paging->items)
			$this->displayAdminItems($rows);
		else
			tooltip::display(
					_("No items found."),
					TOOLTIP_NOTIFICATION);
		
		// As it says, it will display the paging with next/prev page and
		// the page numbers but only if there are more than 1 pages
		// as if its only one there is no reason to show a page number
		$paging->display();
		
		// This is the main form that is used to add/modify items and 
		// if user doesn't have write access to this module we don't 
		// show him the form at all.
		if ($this->userPermissionType & USER_PERMISSION_TYPE_WRITE &&
			(!$this->userPermissionIDs || ($edit && $selected)))
		{
			// If in edit mode we reload the $select as it usually is changed
			// we set the values for the form so the fields in the form will 
			// be prefilled with these values but only if form isn't submitted.
			if ($edit && $selected && ($verifyok || !$form->submitted())) {
				$selected = sql::fetch(sql::run(
					" SELECT * FROM `{examplemoduleitems}`" .
					" WHERE `ID` = '".$id."'"));
				
				$form->setValues($selected);
			}
			
			echo
				"<a name='adminform'></a>";
			
			$form->display();
		}
		
		unset($form);
		
		echo 
			"</div>";	//admin-content
	}
	
	// Adding a new item to the db. Usually it is used only in the admin but you
	// can use it from the client side too.
	function add($values) {
		if (!is_array($values))
			return false;
		
		// If OrderID is not defined we will increment each items order id
		// with one and for the new item we will use the OrderID = 1	
		if ($values['OrderID'] == '') {
			$row = sql::fetch(sql::run(
				" SELECT `OrderID` FROM `{examplemoduleitems}` " .
				" ORDER BY `OrderID` DESC" .
				" LIMIT 1"));
			
			$values['OrderID'] = (int)$row['OrderID']+1;
			
		} else {
			sql::run(
				" UPDATE `{examplemoduleitems}` SET " .
				" `OrderID` = `OrderID` + 1," .
				" `TimeStamp` = `TimeStamp`" .
				" WHERE `OrderID` >= '".(int)$values['OrderID']."'");
		}
		
		// Run sql command and return the new item's id
		$newid = sql::run(
			" INSERT INTO `{examplemoduleitems}` SET ".
			" `Title` = '".
				sql::escape($values['Title'])."'," .
			" `TimeStamp` = " .
				($values['TimeStamp']?
					"'".sql::escape($values['TimeStamp'])."'":
					"NOW()").
				"," .
			" `OrderID` = '".
				(int)$values['OrderID']."'");
			
		if (!$newid) {
			tooltip::display(
				sprintf(_("Item couldn't be created! Error: %s"), 
					sql::error()),
				TOOLTIP_ERROR);
			return false;
		}
		
		return $newid;
	}
	
	// Editing an item, both variables are required but we still do checkings
	function edit($id, $values) {
		if (!(int)$id)
			return false;
		
		// Values has to be an array as we have multiple values for our item
		if (!is_array($values))
			return false;
		
		sql::run(
			" UPDATE `{examplemoduleitems}` SET ".
			" `Title` = '".
				sql::escape($values['Title'])."'," .
			" `TimeStamp` = " .
				($values['TimeStamp']?
					"'".sql::escape($values['TimeStamp'])."'":
					"NOW()").
				"," .
			" `OrderID` = '".
				(int)$values['OrderID']."'" .
			" WHERE `ID` = '".(int)$id."'");
			
		if (sql::affected() == -1) {
			tooltip::display(
				sprintf(_("Item couldn't be updated! Error: %s"), 
					sql::error()),
				TOOLTIP_ERROR);
			return false;
		}
		
		return true;
	}
	
	// A simple delete function
	function delete($id) {
		if (!(int)$id)
			return false;
			
		sql::run(
			" DELETE FROM `{examplemoduleitems}` " .
			" WHERE `ID` = '".(int)$id."'");
		
		return true;
	}
	
	// ************************************************   Client Part
	
	// As the function says this will display all item titles we have
	// in the db. This is only used to demonstrate how to use request
	// that are content codes.
	function displayTitles() {
		echo
			"<div class='example-module-titles'>";
		
		$rows = sql::run(
			" SELECT `Title` FROM `{examplemoduleitems}` " .
			" ORDER BY `OrderID`, `TimeStamp` DESC ");
		
		while($row = sql::fetch($rows)) {
			echo $row['Title'].'; ';
		}
		
		echo
			"</div>";
	}
	
	// This will display one item and is used by the main
	// display() function.
	function displayOne($row, $class = null) {
		echo
			"<div class='example-module-item one ".$class."'>" .
				"<h2 class='example-module-item-title'>" .
					$row['Title'] .
				"</h2>" .
				"<div class='example-module-details comment'>" .
					calendar::datetime($row['TimeStamp']).
				"</div>" .
			"</div>";
	}
	
	// This function is called if there are arguments specified for this
	// form when called the display() function.
	// This only happens if the code was called from the content codes,
	// for e.g. {modules}examplemodule/titles{/modules}
	function displayArguments() {
		if (!$this->arguments)
			return false;
		
		// $this->arguments alwasy containes the text that is written
		// after the module's name in the command, for e.g.
		// in {modules}examplemodule/titles{/modules} $this->arguments = 'titles'
		// You can write anything you want instead 'titles' it's up to you
		// Usually paths are used for this but feel free to use as you wish
		switch(strtolower($this->arguments)) {
			// if argument is titles we display only the titles and we 
			// return true so the script can exit and don't display the rest
			// of the code as usually it would.
			case 'titles':
				$this->displayTitles();
				return true;
			
			// We didn't recognize the argument so we return
			// false so the default code is shown meaning the called
			// display() function below will not exit but display it's content		
			default:
				return false;
		}
	}
	
	function display() {
		// If we were able to display the argument we don't go on with the display
		if ($this->displayArguments())
			return;
		
		// Init our paging class and let it track our own variable
		// so paging can be used with other pagins on a site.
		// This should be always an unique name that isn't/shouldn't
		// be used by any other code in the system
		$paging = new paging($this->limit);
		$paging->track('examplemodulelimit');
		
		// Get the items with the defined limit
		$rows = sql::run(
			" SELECT * FROM `{examplemoduleitems}` " .
			" ORDER BY `OrderID`, `TimeStamp` DESC " .
			" LIMIT ".$paging->limit);
		
		// Again we set the total items for the paging so it knows
		// how many page numbers should be shown.	
		$paging->setTotalItems(sql::count());
		
		// If there are no items at all we don't dump out any data
		// as we keep the site clean.
		if (!sql::rows($rows))
			return;
		
		echo "<div class='example-module'>";
		
		// $i and $total is used to see if we are at the first item at at the last.
		$i = 1;
		$total = sql::rows($rows);
		
		while ($row = sql::fetch($rows)) {
			$class = null;
			if ($i == 1)
				$class .= 'first ';
			if ($i == $total)
				$class .= 'last ';
			
			// We display items one by one and the class is used to
			// tell if it's the last or the first item 
			$this->displayOne($row, $class);
			
			$i++;
		}
		
		// Display paging
		$paging->display();
		
		echo "</div>"; //example-module
	}
}

// And finally registering the module so it is added to the loaded modules list. 
modules::register(
	'exampleModule', //Has to be the same as the name of the class 
	_('Example Module'), //Can be anything but should be no more than two words
	_('This is an example module')); //The description as long as you wish

?>
